﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using DirectShowLib;

namespace vcsharp_audio_playback_demo
{
    enum tState
    {
        EMPTY,
        LOADED,
        PAUSED,
        STOPPED,
        PLAYING
    };
    public partial class Form1 : Form
    {
        private string fileName = string.Empty;
        private IGraphBuilder graphBuilder = null;
        private IMediaControl mediaControl = null;
        private IBasicAudio basicAudio = null;
        private IMediaSeeking mediaSeeking = null;
        private IMediaEvent mediaEvent = null;
        const int minimumVolume = -10000;
        private int volume = 0;
        tState currentState = tState.EMPTY;
        private long stopPosition = 0;
        private long currentPosition = 0;
        public Form1()
        {
            InitializeComponent();
            /// Create graph filter manager to handle the new instance.
            CreateAndInitializeGraph();
        }
        private void openButton_Click(object sender, EventArgs e)
        {
            /// File formats to filter from the open-select window.
            openFileDialog.Filter = "Common Audio Files (.MP3,.WAV,.OGG,.WMA)|*.MP3;*.WAV;*.OGG;*.WMA|All Files (*.*)|*.*";
            DialogResult result = openFileDialog.ShowDialog();
            if (result == DialogResult.OK)
            {
                timer.Enabled = false;
                fileName = openFileDialog.FileName;
                currentState = tState.LOADED;
                MessageBox.Show("File loaded. Press Play button.", "Success");
            }
        }
        private void playButton_Click(object sender, EventArgs e)
        {
            if (fileName == string.Empty || currentState == tState.EMPTY)
            {
                MessageBox.Show("No file selected.", "Error");
                return;
            }
            if (currentState == tState.STOPPED || currentState == tState.LOADED)
            {
                if (currentState != tState.STOPPED)
                {
                    /// Stops all the filters in the filter graph.
                    this.mediaControl.Stop();
                }
                /// Clear graph and all its resources.
                Cleanup();
                /// Create graph filter manager to handle the new instance. 
                CreateAndInitializeGraph();  
                /// Start rendering.
                this.graphBuilder.RenderFile(fileName, null);
                /// Get stop position.
                this.mediaSeeking.GetStopPosition(out stopPosition);
            }
            if (currentState != tState.PLAYING)
            {
                /// Run all the filters in the filter graph.
                this.mediaControl.Run();
                timer.Enabled = true;
                /// Set volume according to volume track bar.
                volumeTrackBar_ValueChanged(sender, e);
                currentState = tState.PLAYING;
            }
        }
        private void pauseButton_Click(object sender, EventArgs e)
        {
            if (currentState == tState.PLAYING)
            {
                timer.Enabled = false;
                /// Pauses all the filters in the filter graph.
                this.mediaControl.Pause();
                currentState = tState.PAUSED;
            }
        }
        private void stopButton_Click(object sender, EventArgs e)
        {
            if (currentState == tState.PLAYING || currentState == tState.PAUSED)
            {
                timer.Enabled = false;
                /// Stops all the filters in the filter graph.
                this.mediaControl.Stop();
                currentState = tState.STOPPED;
            }
        }
        private void timer_Tick_1(object sender, EventArgs e)
        {
            this.mediaSeeking.GetCurrentPosition(out currentPosition);
            if (currentPosition >= stopPosition)
            {
                timer.Enabled = false;
                stopButton_Click(sender, e);
            }
        }
        private void volumeTrackBar_ValueChanged(object sender, EventArgs e)
        {
            if (currentState != tState.EMPTY)
            {
                /// Set volume according to volume track bar.
                volume = (minimumVolume / volumeTrackBar.Value) + 250;
                this.basicAudio.put_Volume(volume);
            }
        }
        /// Create graph filter manager to handle the new instance.
        private void CreateAndInitializeGraph()
        {
            this.graphBuilder = (IGraphBuilder)new FilterGraph();
            this.mediaControl = (IMediaControl)this.graphBuilder;
            this.basicAudio = this.graphBuilder as IBasicAudio;
            this.mediaSeeking = this.graphBuilder as IMediaSeeking;
            this.mediaEvent = this.graphBuilder as IMediaEvent;
        }
        /// Clear graph and all its resources.
        private void Cleanup()
        {
            Marshal.ReleaseComObject(mediaEvent);
            Marshal.ReleaseComObject(mediaSeeking);
            Marshal.ReleaseComObject(basicAudio);
            Marshal.ReleaseComObject(mediaControl);
            Marshal.ReleaseComObject(graphBuilder);
        }
        private void Form1_closing(object sender, EventArgs e)
        {
            /// Clear graph and all its resources.
            Cleanup();
        }
    }       
}